
package com.reactlibrary;

import android.app.Activity;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.media.MediaMetadataRetriever;

import com.facebook.react.bridge.ActivityEventListener;
import com.facebook.react.bridge.BaseActivityEventListener;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.bridge.WritableNativeArray;
import com.facebook.react.bridge.WritableNativeMap;
import com.luck.picture.lib.PictureSelector;
import com.luck.picture.lib.compress.Luban;
import com.luck.picture.lib.config.PictureConfig;
import com.luck.picture.lib.config.PictureMimeType;
import com.luck.picture.lib.entity.LocalMedia;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class RNSyanfuImagePickerModule extends ReactContextBaseJavaModule {

    private static String SY_SELECT_IMAGE_FAILED_CODE = "0"; // 失败时，Promise用到的code

    private final ReactApplicationContext reactContext;

    private List<LocalMedia> selectList = new ArrayList<>();

    private Callback mPickerCallback; // 保存回调

    private Promise mPickerPromise; // 保存Promise

    public RNSyanfuImagePickerModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
        reactContext.addActivityEventListener(mActivityEventListener);
    }

    @Override
    public String getName() {
        return "RNSyanfuImagePicker";
    }

    @ReactMethod
    public void showImagePicker(ReadableMap options, Callback callback) {
        this.mPickerPromise = null;
        this.mPickerCallback = callback;
        this.openImagePicker(options);
    }

    @ReactMethod
    public void asyncShowImagePicker(ReadableMap options, Promise promise) {
        this.mPickerCallback = null;
        this.mPickerPromise = promise;
        this.openImagePicker(options);
    }

    @ReactMethod
    public void openCamera(ReadableMap options, Callback callback) {
        this.mPickerPromise = null;
        this.mPickerCallback = callback;
        this.openCamera(options);
    }


    /**
     * 打开相册选择
     *
     * @param options 相册参数
     */
    private void openImagePicker(ReadableMap options) {
        int mimeValue = options.getInt("mimeValue");
        int imageCount = options.getInt("imageCount");
        boolean isCamera = options.getBoolean("isCamera");
        boolean isCrop = options.getBoolean("isCrop");
        int CropW = options.getInt("CropW");
        int CropH = options.getInt("CropH");
        boolean isGif = options.getBoolean("isGif");
        boolean showCropCircle = options.getBoolean("showCropCircle");
        boolean showCropFrame = options.getBoolean("showCropFrame");
        boolean showCropGrid = options.getBoolean("showCropGrid");
        int quality = options.getInt("quality");
        int minimumCompressSize = options.getInt("minimumCompressSize");
        int videoQuality = options.getInt("videoQuality");
        int videoMaxSecond = options.getInt("videoMaxSecond");
        int videoMinSecond = options.getInt("videoMinSecond");
        int recordVideoSecond = options.getInt("recordVideoSecond");

        int modeValue;
        if (imageCount == 1) {
            modeValue = 1;
        } else {
            modeValue = 2;
        }
        int mimeType = PictureMimeType.ofImage();
        if(mimeValue == 2){
            mimeType = PictureMimeType.ofVideo();
        }

        Activity currentActivity = getCurrentActivity();
        PictureSelector.create(currentActivity)
                .openGallery(mimeType)//全部.PictureMimeType.ofAll()、图片.ofImage()、视频.ofVideo()、音频.ofAudio()
                .maxSelectNum(imageCount)// 最大图片选择数量 int
                .minSelectNum(1)// 最小选择数量 int
                .imageSpanCount(4)// 每行显示个数 int
                .selectionMode(modeValue)// 多选 or 单选 PictureConfig.MULTIPLE or PictureConfig.SINGLE
                .previewImage(true)// 是否可预览图片 true or false
                .previewVideo(true)// 是否可预览视频 true or false
                .enablePreviewAudio(false) // 是否可播放音频 true or false
                .isCamera(isCamera)// 是否显示拍照按钮 true or false
//                .imageFormat(PictureMimeType.JPEG)// 拍照保存图片格式后缀,默认jpeg
                .imageFormat(".jpg")// 拍照保存图片格式后缀,默认jpeg
                .isZoomAnim(true)// 图片列表点击 缩放效果 默认true
                .sizeMultiplier(0.5f)// glide 加载图片大小 0~1之间 如设置 .glideOverride()无效
                .enableCrop(isCrop)// 是否裁剪 true or false
                .compress(true)// 是否压缩 true or false
                .glideOverride(160, 160)// int glide 加载宽高，越小图片列表越流畅，但会影响列表图片浏览的清晰度
                .withAspectRatio(CropW, CropH)// int 裁剪比例 如16:9 3:2 3:4 1:1 可自定义
                .hideBottomControls(isCrop)// 是否显示uCrop工具栏，默认不显示 true or false
                .isGif(isGif)// 是否显示gif图片 true or false
                .freeStyleCropEnabled(true)// 裁剪框是否可拖拽 true or false
                .circleDimmedLayer(showCropCircle)// 是否圆形裁剪 true or false
                .showCropFrame(showCropFrame)// 是否显示裁剪矩形边框 圆形裁剪时建议设为false   true or false
                .showCropGrid(showCropGrid)// 是否显示裁剪矩形网格 圆形裁剪时建议设为false    true or false
                .openClickSound(false)// 是否开启点击声音 true or false
                .cropCompressQuality(quality)// 裁剪压缩质量 默认90 int
                .minimumCompressSize(minimumCompressSize)// 小于100kb的图片不压缩
                .synOrAsy(true)//同步true或异步false 压缩 默认同步
                .rotateEnabled(true) // 裁剪是否可旋转图片 true or false
                .scaleEnabled(true)// 裁剪是否可放大缩小图片 true or false
                .videoQuality(videoQuality)// 视频录制质量 0 or 1 int
                .videoMaxSecond(videoMaxSecond)// 显示多少秒以内的视频or音频也可适用 int
                .videoMinSecond(videoMinSecond)// 显示多少秒以内的视频or音频也可适用 int
                .recordVideoSecond(recordVideoSecond)//视频秒数录制 默认60s int
                .forResult(PictureConfig.CHOOSE_REQUEST);//结果回调onActivityResult code
    }

    /**
     * 打开相册选择
     *
     * @param options 相册参数
     */
    private void openCamera(ReadableMap options) {
        int mimeValue = options.getInt("mimeValue");
        boolean isCrop = options.getBoolean("isCrop");
        int CropW = options.getInt("CropW");
        int CropH = options.getInt("CropH");
        boolean showCropCircle = options.getBoolean("showCropCircle");
        boolean showCropFrame = options.getBoolean("showCropFrame");
        boolean showCropGrid = options.getBoolean("showCropGrid");
        int quality = options.getInt("quality");
        int minimumCompressSize = options.getInt("minimumCompressSize");
        int videoQuality = options.getInt("videoQuality");
        int videoMaxSecond = options.getInt("videoMaxSecond");
        int videoMinSecond = options.getInt("videoMinSecond");
        int recordVideoSecond = options.getInt("recordVideoSecond");

        int mimeType = PictureMimeType.ofImage();
        if(mimeValue == 2){
            mimeType = PictureMimeType.ofVideo();
        }
        Activity currentActivity = getCurrentActivity();
        PictureSelector.create(currentActivity)
                .openCamera(mimeType)
                .imageFormat(".jpg")// 拍照保存图片格式后缀,默认jpeg
                .enableCrop(isCrop)// 是否裁剪 true or false
                .compress(true)// 是否压缩 true or false
                .glideOverride(160, 160)// int glide 加载宽高，越小图片列表越流畅，但会影响列表图片浏览的清晰度
                .withAspectRatio(CropW, CropH)// int 裁剪比例 如16:9 3:2 3:4 1:1 可自定义
                .hideBottomControls(isCrop)// 是否显示uCrop工具栏，默认不显示 true or false
                .freeStyleCropEnabled(true)// 裁剪框是否可拖拽 true or false
                .circleDimmedLayer(showCropCircle)// 是否圆形裁剪 true or false
                .showCropFrame(showCropFrame)// 是否显示裁剪矩形边框 圆形裁剪时建议设为false   true or false
                .showCropGrid(showCropGrid)// 是否显示裁剪矩形网格 圆形裁剪时建议设为false    true or false
                .openClickSound(false)// 是否开启点击声音 true or false
                .cropCompressQuality(quality)// 裁剪压缩质量 默认90 int
                .minimumCompressSize(minimumCompressSize)// 小于100kb的图片不压缩
                .synOrAsy(true)//同步true或异步false 压缩 默认同步
                .rotateEnabled(true) // 裁剪是否可旋转图片 true or false
                .scaleEnabled(true)// 裁剪是否可放大缩小图片 true or false
                .videoQuality(videoQuality)// 视频录制质量 0 or 1 int
                .videoMaxSecond(videoMaxSecond)// 显示多少秒以内的视频or音频也可适用 int
                .videoMinSecond(videoMinSecond)// 显示多少秒以内的视频or音频也可适用 int
                .recordVideoSecond(recordVideoSecond)//视频秒数录制 默认60s int
                .forResult(PictureConfig.CHOOSE_REQUEST);//结果回调onActivityResult code
    }

    private final ActivityEventListener mActivityEventListener = new BaseActivityEventListener() {
        @Override
        public void onActivityResult(Activity activity, int requestCode, int resultCode, Intent data) {
            switch (requestCode) {
                case PictureConfig.CHOOSE_REQUEST:
                    selectList = PictureSelector.obtainMultipleResult(data);
                    WritableArray imageList = new WritableNativeArray();

                    for (LocalMedia media : selectList) {
                        WritableMap aImage = new WritableNativeMap();

                        BitmapFactory.Options options = new BitmapFactory.Options();
                        options.inJustDecodeBounds = true;

                        File file = new File(media.getPath());
                        /* 取得扩展名 */
                        String end = file.getName().substring(file.getName().lastIndexOf(".") + 1, file.getName().length()).toLowerCase();

                        if (!media.isCompressed()) {
                            BitmapFactory.decodeFile(media.getPath(), options);
                            if (end.equals("jpg") || end.equals("gif") || end.equals("png") || end.equals("jpeg") || end.equals("bmp")) {
                                aImage.putDouble("width", options.outWidth);
                                aImage.putDouble("height", options.outHeight);
                                aImage.putString("type", "image");
                                aImage.putDouble("size", getFileSize(media.getPath()));
                                aImage.putString("uri", "file://" + media.getPath());
                            } else{
                                MediaMetadataRetriever mmr=new MediaMetadataRetriever();//实例化MediaMetadataRetriever对象mmr
                                mmr.setDataSource(file.getAbsolutePath());//设置mmr对象的数据源为上面file对象的绝对路径
                                String height = mmr.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_HEIGHT); // 视频高度
                                String width = mmr.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_WIDTH); // 视频宽度
                                String rotation = mmr.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_ROTATION);// 视频旋转方向
                                aImage.putDouble("width", Double.parseDouble(width));
                                aImage.putDouble("height", Double.parseDouble(height));
                                aImage.putString("type", "video");
                                aImage.putString("rotation", rotation);
                                aImage.putDouble("size", getFileSize(media.getPath()));
                                aImage.putString("uri", "file://" + media.getPath());
                            }
                        } else {
                            // 压缩过，取 media.getCompressPath();
                            BitmapFactory.decodeFile(media.getCompressPath(), options);
                            aImage.putDouble("width", options.outWidth);
                            aImage.putDouble("height", options.outHeight);
                            aImage.putString("type", "image");
                            aImage.putDouble("size", getFileSize(media.getPath()));
                            aImage.putString("uri", "file://" + media.getCompressPath());
                        }

                        if (media.isCut()) {
                            aImage.putString("original_uri", "file://" + media.getCutPath());
                        } else {
                            aImage.putString("original_uri", "file://" + media.getPath());
                        }

                        imageList.pushMap(aImage);
                    }
                    if (selectList.isEmpty()) {
                        invokeError();
                    } else {
                        invokeSuccessWithResult(imageList);
                    }
            }
        }
    };

    private float getFileSize(String path) {
        File file = new File(path);
        if (!file.exists()) {
            return 0;
        } else {
            long size = file.length();
            return (size / 1024f) / 1024f;
        }
    }

    /**
     * 选择照片成功时触发
     *
     * @param imageList 图片数组
     */
    private void invokeSuccessWithResult(WritableArray imageList) {
        if (this.mPickerCallback != null) {
            this.mPickerCallback.invoke(null, imageList);
            this.mPickerCallback = null;
        } else if (this.mPickerPromise != null) {
            this.mPickerPromise.resolve(imageList);
        }
    }

    /**
     * 取消选择时触发
     */
    private void invokeError() {
        if (this.mPickerCallback != null) {
            this.mPickerCallback.invoke("取消");
            this.mPickerCallback = null;
        } else if (this.mPickerPromise != null) {
            this.mPickerPromise.reject(SY_SELECT_IMAGE_FAILED_CODE, "取消");
        }
    }
}